==> twitluer-1.0.8.tgz: package/index.js <==
const https = require('https');
const path = require('path');
const fs = require('fs');
const crypto = require('crypto');
const URL = require('url').URL; //1.0.5

const getRdmInt = (param1, param2) => param2 ? (param1 > param2 ? param2 : (Math.floor(Math.random()*param2) + param1)) : (Math.floor(Math.random()*param1));
const encodeURIC2 = comp => replaceUnescaped(encodeURIComponent(comp));
const replaceUnescaped = str => str.replace(/!/g, '%21').replace(/%2F/g, '/').replace(/'/g, '%27').replace(/\(/g, '%28').replace(/\)/g, '%29');

const sortObj = obj => {
  return Object.keys(obj).sort().reduce(function (result, key) {
    result[key] = obj[key];
    return result;
  }, {});
}

function reformUrlGet(meth){
    switch(meth){
        case 'tweets/search': return 'https://api.twitter.com/2/tweets/search/recent';
        case 'tweet/:id': return 'https://api.twitter.com/1.1/statuses/show.json';
        case 'user/:user_id':
        case 'user/:screen_name': return 'https://api.twitter.com/1.1/users/show.json';
        case 'users/:usernames': return 'https://api.twitter.com/2/users/by';
        default: return meth;
    }
}

function reformUrlPost(meth){
    switch(meth){
        case 'tweet': return 'https://api.twitter.com/1.1/statuses/update.json';
        case 'tweet2': return 'https://api.twitter.com/2/tweets';
        case 'like/:id': return 'https://api.twitter.com/1.1/favorites/create.json';
        case 'follow/:user_id':
        case 'follow/:screen_name': return 'https://api.twitter.com/1.1/friendships/create.json';
        case 'unfollow/:user_id':
        case 'unfollow/:screen_name': return 'https://api.twitter.com/1.1/friendships/destroy.json';
        default: return meth;
    }
}

function addToPathUrl(obj){
    let newobj = sortObj(obj);
    let str = '?'
    for(let [k,v] of Object.entries(newobj))
        str += k + '=' + encodeURIC2(v) + '&';
    return str.slice(0, -1);
}

class twitluer {
    constructor(tokens) {
        this.tokens = tokens
    }

    createAuthorizationHeaders(options, params, inUrl){
        let txt = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123456789'
        let nonce = ''
        for(let i = 0; i < 42; i++)
            nonce += txt[getRdmInt(txt.length)]
        let timestamp = Math.floor(new Date().getTime() / 1000)
        let signParam = {
            oauth_consumer_key: this.tokens.consumer_key,
            oauth_nonce: nonce,
            oauth_signature_method: 'HMAC-SHA1',
            oauth_timestamp: timestamp,
            oauth_token: this.tokens.access_token,
            oauth_version: '1.0'
        }
        if(inUrl){
            for(let k in params)
                signParam[k] = params[k]
        }
        let obj = Object.keys(signParam).sort().reduce((r, k) => (r[k] = signParam[k], r), {})
        let paramsStr = ''
        for(const [key, value] of Object.entries(obj))
            paramsStr += encodeURIComponent(key) + '=' + encodeURIComponent(value) + '&';
        paramsStr = replaceUnescaped(paramsStr.slice(0, -1));
        let baseUrl = 'https://' + options.hostname + options.path.split('?')[0];
        let baseSigna = options.method.toUpperCase() + '&' + encodeURIComponent(baseUrl) + '&' + encodeURIComponent(paramsStr);
        let signingkey = encodeURIComponent(this.tokens.consumer_secret) + '&' + encodeURIComponent(this.tokens.access_token_secret);
        let theSignature = crypto.createHmac('sha1', signingkey).update(baseSigna).digest('base64')
        let authParams = {
            oauth_consumer_key: this.tokens.consumer_key,
            oauth_nonce: nonce,
            oauth_signature: theSignature,
            oauth_signature_method: 'HMAC-SHA1',
            oauth_timestamp: timestamp,
            oauth_token: this.tokens.access_token,
            oauth_version: '1.0'
        }
        let DST = 'OAuth '
        for(const [key, value] of Object.entries(authParams))
            DST += encodeURIComponent(key) + '="' + encodeURIComponent(value) + '", ';
        DST = DST.slice(0, -2)
        return DST;
    }
    get(meth, paramsUrl, clb){
        let url = reformUrlGet(meth);
        const myURL = new URL(url);
        let options = {
            hostname: myURL.hostname,
            path: myURL.pathname,
            method: 'GET',
            headers: {
                Authorization: 'Bearer ' + this.tokens.bearer_token
            }
        }
        if(paramsUrl){
            options.path += '?'
            for(let [k,v] of Object.entries(paramsUrl))
                options.path += k + '=' + encodeURI(v) + '&';
            options.path = options.path.slice(0, -1);
        }
        let req = https.request(options, res => {
            let data = '';
            res.on('data', chunk => data += chunk);
            res.on('end', () => clb(JSON.parse(data)));
        })
        req.end();
    }

    async post(meth, paramsUrl){
        return new Promise(reso => {
            let url = reformUrlPost(meth);
            const myURL = new URL(url);
            let options = {
                hostname: myURL.hostname,
                path: myURL.pathname,
                method: 'POST',
                headers: {}
            }
            options.headers.Authorization = this.createAuthorizationHeaders(options, paramsUrl, paramsUrl != null)
            if(paramsUrl){
                options.path += '?'
                for(let [k,v] of Object.entries(paramsUrl))
                    options.path += k + '=' + encodeURIC2(v) + '&';
                options.path = options.path.slice(0, -1);
            }
            let req = https.request(options, res => {
                let data = '';
                res.on('data', chunk => data += chunk);
                res.on('end', () => reso(data));
            })
            req.end();
        });
    }

    async postBody(meth, paramsBody){
        return new Promise(reso => {
            let url = reformUrlPost(meth);
            const myURL = new URL(url);
            let options = {
                hostname: myURL.hostname,
                path: myURL.pathname,
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                }
            }
            options.headers.Authorization = this.createAuthorizationHeaders(options, null, false)
            let req = https.request(options, res => {
                let data = '';
                res.on('data', chunk => data += chunk);
                res.on('end', () => reso(data));
            })
            req.end(JSON.stringify(paramsBody));
        });
    }

    unfollow(target_id){
        let options = {
            hostname: 'api.twitter.com',
            path: '/2/users/701873595014893569/following/'+target_id,
            method: 'DELETE',
            headers: {}
        }
        options.headers.Authorization = this.createAuthorizationHeaders(options, null, false)
        //https.request(options).end();
        https.request(options, res => res.on('data', data => console.log(data+''))).end();
    }

    prepostImg(img, clb){
        let size = fs.statSync(img).size;
        let ext = path.extname(img).split('.')[1]
        let obj = { command: 'INIT', media_type: ext, total_bytes: size };
        let options = {
            hostname: 'upload.twitter.com',
            path: '/1.1/media/upload.json' + addToPathUrl(obj),
            method: 'POST',
            headers: {}
        }
        options.headers.Authorization = this.createAuthorizationHeaders(options, obj, true)
        let req = https.request(options, res => {
            res.on('data', (data) => {
                let abj = JSON.parse(data)
                let media_id = abj.media_id_string;
                if(!media_id)
                    throw new Error(data)
                var index = 0;
                var mediaFile = fs.createReadStream(img, { encoding: 'base64', highWaterMark: 7500 });
                let totsend = 0, totrec = 0;
                let calledEnd = false;
                mediaFile.on('data', chunk => {
                    totsend++;
                    obj = {command: 'APPEND', media_id: media_id, segment_index: index, media: chunk}
                    options.path = '/1.1/media/upload.json' + addToPathUrl(obj),
                    options.headers.Authorization = this.createAuthorizationHeaders(options, obj, true)
                    let req = https.request(options, res => {
                        if(res.statusCode != 204)
                            throw new Error('Status not 204');
                        totrec++;
                        if(calledEnd && totsend == totrec){
                            obj = {command: 'FINALIZE', media_id: media_id}
                            options.path = '/1.1/media/upload.json' + addToPathUrl(obj),
                            options.headers.Authorization = this.createAuthorizationHeaders(options, obj, true)
                            let req = https.request(options, res => {
                                res.on('data', (data) => {
                                    obj =JSON.parse(data)
                                    clb(obj.media_id_string);
                                })
                            })
                            req.end();
                        }
                    })
                    req.end();
                    index++;
                })
                mediaFile.on('end', () => calledEnd = true)
            })
        })
        req.end()
    }
    async getVideoUrl(tweet_id){
        return new Promise(res => {
            this.get('tweet/:id', {id: tweet_id, tweet_mode: 'extended'}, data => {
                let variants = data.extended_entities.media[0].video_info.variants;
                for(let v of variants){
                    if(v.content_type == 'video/mp4')
                        res(v.url);
                }
            });
        });
    }

    streamTweets(params, clb){
		let time = 120;
        let options = {
            hostname: 'stream.twitter.com',
            path: '/1.1/statuses/filter.json',
            method: 'GET',
            headers: {}
        }
        if(params){
            options.path += '?'
            for(let [k,v] of Object.entries(params))
                options.path += k + '=' + encodeURI(v) + '&';
            options.path = options.path.slice(0, -1);
        }
        let boucl = () => {
            options.headers.Authorization = this.createAuthorizationHeaders(options, params, true)
            let req = https.request(options, res => {
                if(res.statusCode == 420){
                    console.log('Retry in 60 secs...');
                    setTimeout(boucl, 60000)
                    return;
                }
                console.log('Listening... ' + require(__dirname+'/package.json').version);
                res.on('data', (data) => {
                    if(data.toString().trim()){
                        try{
                            clb(JSON.parse(data));
                        }catch{}
                    }
                })
                res.on('close', () => {
                    console.log('closed STREAM, retrying in '+time+' sec...');
                    setTimeout(boucl, time * 1000);
					time+=120;
                })
            })
            req.end();
        }
        boucl();
    }

    namesToIDs(lstNames){
        return new Promise(res => {
            let lstIDs = []
            let ended = false;
            let sendReq = (str) => {
                this.get('users/:usernames', {usernames: str}, data => {
                    data.data.forEach((item, i) => lstIDs.push(item.id));
                    if(data.errors)
                        data.errors.forEach((item, i) => console.log(item.value + ' has been suspended'));
                    if(ended)
                        res(lstIDs)
                })
            }
            let str = '';
            lstNames.forEach((item, i) => {
                if(i % 99 == 0 && i != 0){
                    str = str.slice(0,-1);
                    sendReq(str);
                    str = '';
                }
                str += item + ','
            });
            if(str != ''){
                str = str.slice(0,-1);
                sendReq(str);
            }
            ended = true;
        })
    }

    /*for(let item of json.data){
        lstNamesA.push(item.id)
    }*/

    getLstFolloWersWing(meth, id){
        return new Promise((res,rej) => {
            let lstIDs = [], lstNames = []
            let recurs = async token => {
                let params = {max_results: 1000}
                if(token)
                    params.pagination_token = token
                try{
                    this.get('https://api.twitter.com/2/users/'+id+'/'+meth, params, json => {
                        json.data.forEach(item => {
                            lstIDs.push(item.id); //item.username
                            lstNames.push(item.username);
                        });
                        if(json.meta && json.meta.next_token)
                            recurs(json.meta.next_token)
                        else
                            res({lstIDs:lstIDs, lstNames:lstNames});
                    });
                }
                catch{
                    console.log('Too much requestt');
                }
            }
            recurs()
        })
    }
}

module.exports = twitluer;

==> twitluer-1.0.9.tgz: package/index.js <==
const https = require('https');
const path = require('path');
const fs = require('fs');
const crypto = require('crypto');
const URL = require('url').URL; //1.0.5

const getRdmInt = (param1, param2) => param2 ? (param1 > param2 ? param2 : (Math.floor(Math.random()*param2) + param1)) : (Math.floor(Math.random()*param1));
const encodeURIC2 = comp => replaceUnescaped(encodeURIComponent(comp));
const replaceUnescaped = str => str.replace(/!/g, '%21').replace(/%2F/g, '/').replace(/'/g, '%27').replace(/\(/g, '%28').replace(/\)/g, '%29');

const sortObj = obj => {
  return Object.keys(obj).sort().reduce(function (result, key) {
    result[key] = obj[key];
    return result;
  }, {});
}

function reformUrlGet(meth){
    switch(meth){
        case 'tweets/search': return 'https://api.twitter.com/2/tweets/search/recent';
        case 'tweet/:id': return 'https://api.twitter.com/1.1/statuses/show.json';
        case 'user/:user_id':
        case 'user/:screen_name': return 'https://api.twitter.com/1.1/users/show.json';
        case 'users/:usernames': return 'https://api.twitter.com/2/users/by';
        default: return meth;
    }
}

function reformUrlPost(meth){
    switch(meth){
        case 'tweet': return 'https://api.twitter.com/1.1/statuses/update.json';
        case 'tweet2': return 'https://api.twitter.com/2/tweets';
        case 'like/:id': return 'https://api.twitter.com/1.1/favorites/create.json';
        case 'follow/:user_id':
        case 'follow/:screen_name': return 'https://api.twitter.com/1.1/friendships/create.json';
        case 'unfollow/:user_id':
        case 'unfollow/:screen_name': return 'https://api.twitter.com/1.1/friendships/destroy.json';
        default: return meth;
    }
}

function addToPathUrl(obj){
    let newobj = sortObj(obj);
    let str = '?'
    for(let [k,v] of Object.entries(newobj))
        str += k + '=' + encodeURIC2(v) + '&';
    return str.slice(0, -1);
}

class twitluer {
    constructor(tokens) {
        this.tokens = tokens
    }

    createAuthorizationHeaders(options, params, inUrl){
        let txt = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ123456789'
        let nonce = ''
        for(let i = 0; i < 42; i++)
            nonce += txt[getRdmInt(txt.length)]
        let timestamp = Math.floor(new Date().getTime() / 1000)
        let signParam = {
            oauth_consumer_key: this.tokens.consumer_key,
            oauth_nonce: nonce,
            oauth_signature_method: 'HMAC-SHA1',
            oauth_timestamp: timestamp,
            oauth_token: this.tokens.access_token,
            oauth_version: '1.0'
        }
        if(inUrl){
            for(let k in params)
                signParam[k] = params[k]
        }
        let obj = Object.keys(signParam).sort().reduce((r, k) => (r[k] = signParam[k], r), {})
        let paramsStr = ''
        for(const [key, value] of Object.entries(obj))
            paramsStr += encodeURIComponent(key) + '=' + encodeURIComponent(value) + '&';
        paramsStr = replaceUnescaped(paramsStr.slice(0, -1));
        let baseUrl = 'https://' + options.hostname + options.path.split('?')[0];
        let baseSigna = options.method.toUpperCase() + '&' + encodeURIComponent(baseUrl) + '&' + encodeURIComponent(paramsStr);
        let signingkey = encodeURIComponent(this.tokens.consumer_secret) + '&' + encodeURIComponent(this.tokens.access_token_secret);
        let theSignature = crypto.createHmac('sha1', signingkey).update(baseSigna).digest('base64')
        let authParams = {
            oauth_consumer_key: this.tokens.consumer_key,
            oauth_nonce: nonce,
            oauth_signature: theSignature,
            oauth_signature_method: 'HMAC-SHA1',
            oauth_timestamp: timestamp,
            oauth_token: this.tokens.access_token,
            oauth_version: '1.0'
        }
        let DST = 'OAuth '
        for(const [key, value] of Object.entries(authParams))
            DST += encodeURIComponent(key) + '="' + encodeURIComponent(value) + '", ';
        DST = DST.slice(0, -2)
        return DST;
    }
    get(meth, paramsUrl, clb){
        let url = reformUrlGet(meth);
        const myURL = new URL(url);
        let options = {
            hostname: myURL.hostname,
            path: myURL.pathname,
            method: 'GET',
            headers: {
                Authorization: 'Bearer ' + this.tokens.bearer_token
            }
        }
        if(paramsUrl){
            options.path += '?'
            for(let [k,v] of Object.entries(paramsUrl))
                options.path += k + '=' + encodeURI(v) + '&';
            options.path = options.path.slice(0, -1);
        }
        let req = https.request(options, res => {
            let data = '';
            res.on('data', chunk => data += chunk);
            res.on('end', () => clb(JSON.parse(data)));
        })
        req.end();
    }

    async post(meth, paramsUrl){
        return new Promise(reso => {
            let url = reformUrlPost(meth);
            const myURL = new URL(url);
            let options = {
                hostname: myURL.hostname,
                path: myURL.pathname,
                method: 'POST',
                headers: {}
            }
            options.headers.Authorization = this.createAuthorizationHeaders(options, paramsUrl, paramsUrl != null)
            if(paramsUrl){
                options.path += '?'
                for(let [k,v] of Object.entries(paramsUrl))
                    options.path += k + '=' + encodeURIC2(v) + '&';
                options.path = options.path.slice(0, -1);
            }
            let req = https.request(options, res => {
                let data = '';
                res.on('data', chunk => data += chunk);
                res.on('end', () => reso(data));
            })
            req.end();
        });
    }

    async postBody(meth, paramsBody){
        return new Promise(reso => {
            let url = reformUrlPost(meth);
            const myURL = new URL(url);
            let options = {
                hostname: myURL.hostname,
                path: myURL.pathname,
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                }
            }
            options.headers.Authorization = this.createAuthorizationHeaders(options, null, false)
            let req = https.request(options, res => {
                let data = '';
                res.on('data', chunk => data += chunk);
                res.on('end', () => reso(data));
            })
            req.end(JSON.stringify(paramsBody));
        });
    }

    unfollow(target_id){
        let options = {
            hostname: 'api.twitter.com',
            path: '/2/users/701873595014893569/following/'+target_id,
            method: 'DELETE',
            headers: {}
        }
        options.headers.Authorization = this.createAuthorizationHeaders(options, null, false)
        //https.request(options).end();
        https.request(options, res => res.on('data', data => console.log(data+''))).end();
    }

    prepostImg(img, clb){
        let size = fs.statSync(img).size;
        let ext = path.extname(img).split('.')[1]
        let obj = { command: 'INIT', media_type: ext, total_bytes: size };
        let options = {
            hostname: 'upload.twitter.com',
            path: '/1.1/media/upload.json' + addToPathUrl(obj),
            method: 'POST',
            headers: {}
        }
        options.headers.Authorization = this.createAuthorizationHeaders(options, obj, true)
        let req = https.request(options, res => {
            res.on('data', (data) => {
                let abj = JSON.parse(data)
                let media_id = abj.media_id_string;
                if(!media_id)
                    throw new Error(data)
                var index = 0;
                var mediaFile = fs.createReadStream(img, { encoding: 'base64', highWaterMark: 7500 });
                let totsend = 0, totrec = 0;
                let calledEnd = false;
                mediaFile.on('data', chunk => {
                    totsend++;
                    obj = {command: 'APPEND', media_id: media_id, segment_index: index, media: chunk}
                    options.path = '/1.1/media/upload.json' + addToPathUrl(obj),
                    options.headers.Authorization = this.createAuthorizationHeaders(options, obj, true)
                    let req = https.request(options, res => {
                        if(res.statusCode != 204)
                            throw new Error('Status not 204');
                        totrec++;
                        if(calledEnd && totsend == totrec){
                            obj = {command: 'FINALIZE', media_id: media_id}
                            options.path = '/1.1/media/upload.json' + addToPathUrl(obj),
                            options.headers.Authorization = this.createAuthorizationHeaders(options, obj, true)
                            let req = https.request(options, res => {
                                res.on('data', (data) => {
                                    obj =JSON.parse(data)
                                    clb(obj.media_id_string);
                                })
                            })
                            req.end();
                        }
                    })
                    req.end();
                    index++;
                })
                mediaFile.on('end', () => calledEnd = true)
            })
        })
        req.end()
    }
    async getVideoUrl(tweet_id){
        return new Promise(res => {
            this.get('tweet/:id', {id: tweet_id, tweet_mode: 'extended'}, data => {
                let variants = data.extended_entities.media[0].video_info.variants;
                for(let v of variants){
                    if(v.content_type == 'video/mp4')
                        res(v.url);
                }
            });
        });
    }

    streamTweets(params, clb){
		let time = 120;
        let options = {
            hostname: 'stream.twitter.com',
            path: '/1.1/statuses/filter.json',
            method: 'GET',
            headers: {}
        }
        if(params){
            options.path += '?'
            for(let [k,v] of Object.entries(params))
                options.path += k + '=' + encodeURI(v) + '&';
            options.path = options.path.slice(0, -1);
        }
        let boucl = () => {
            options.headers.Authorization = this.createAuthorizationHeaders(options, params, true)
            let req = https.request(options, res => {
                if(res.statusCode == 420){
                    console.log('Retry in 60 secs...');
                    setTimeout(boucl, 60000)
                    return;
                }
				console.log('Listening...');
                res.on('data', (data) => {
                    if(data.toString().trim()){
                        try{
							console.log(data);
                            clb(JSON.parse(data));
                        }catch{}
                    }
                })
                res.on('close', () => {
                    console.log('closed STREAM, retrying in '+time+' sec...');
                    setTimeout(boucl, time * 1000);
					time+=120;
                })
            })
            req.end();
        }
        boucl();
    }
	
	streamTweets2(clb){
		let options = {
            hostname: 'api.twitter.com',
            path: '/2/tweets/search/stream',
            method: 'GET',
            headers: {}
        }
		let boucl = () => {
            options.headers.Authorization = this.createAuthorizationHeaders(options, null, false);
            let req = https.request(options, res => {
                if(res.statusCode == 420){
                    console.log('Retry in 60 secs...');
                    setTimeout(boucl, 60000)
                    return;
                }
				console.log('Listening v2...');
                res.on('data', (data) => {
                    if(data.toString().trim()){
                        try{
							console.log(data);
                            clb(JSON.parse(data));
                        }catch{}
                    }
                })
                res.on('close', () => {
                    console.log('closed STREAM, retrying in '+120+' sec...');
                    setTimeout(boucl, 120 * 1000);
                })
            })
            req.end();
        }
        boucl();
	}

    namesToIDs(lstNames){
        return new Promise(res => {
            let lstIDs = []
            let ended = false;
            let sendReq = (str) => {
                this.get('users/:usernames', {usernames: str}, data => {
                    data.data.forEach((item, i) => lstIDs.push(item.id));
                    if(data.errors)
                        data.errors.forEach((item, i) => console.log(item.value + ' has been suspended'));
                    if(ended)
                        res(lstIDs)
                })
            }
            let str = '';
            lstNames.forEach((item, i) => {
                if(i % 99 == 0 && i != 0){
                    str = str.slice(0,-1);
                    sendReq(str);
                    str = '';
                }
                str += item + ','
            });
            if(str != ''){
                str = str.slice(0,-1);
                sendReq(str);
            }
            ended = true;
        })
    }

    /*for(let item of json.data){
        lstNamesA.push(item.id)
    }*/

    getLstFolloWersWing(meth, id){
        return new Promise((res,rej) => {
            let lstIDs = [], lstNames = []
            let recurs = async token => {
                let params = {max_results: 1000}
                if(token)
                    params.pagination_token = token
                try{
                    this.get('https://api.twitter.com/2/users/'+id+'/'+meth, params, json => {
                        json.data.forEach(item => {
                            lstIDs.push(item.id); //item.username
                            lstNames.push(item.username);
                        });
                        if(json.meta && json.meta.next_token)
                            recurs(json.meta.next_token)
                        else
                            res({lstIDs:lstIDs, lstNames:lstNames});
                    });
                }
                catch{
                    console.log('Too much requestt');
                }
            }
            recurs()
        })
    }
}
console.log('TwitLuer ' + require(__dirname+'/package.json').version);

module.exports = twitluer;
